@@ -1,7 +1,3 @@
-# -*- mode: ruby; coding: us-ascii; -*-
-
-require "rubygems"
-
 begin
   gem "rake"
 rescue Gem::LoadError
@@ -91,7 +87,7 @@ class Hoe
   include Rake::DSL if defined?(Rake::DSL)
 
   # duh
-  VERSION = "3.17.1"
+  VERSION = "3.22.3"
 
   @@plugins = [:clean, :debug, :deps, :flay, :flog, :newb, :package,
                :publish, :gemcutter, :signing, :test]
@@ -111,6 +107,21 @@ class Hoe
 
   RUBY_FLAGS = ENV["RUBY_FLAGS"] || default_ruby_flags
 
+  ##
+  # Map from the commonly used url names to gemspec's metadata keys
+  # See https://guides.rubygems.org/specification-reference/#metadata
+
+  URLS_TO_META_MAP = {
+    "bugs" => "bug_tracker_uri",
+    "clog" => "changelog_uri",
+    "doco" => "documentation_uri",
+    "docs" => "documentation_uri",
+    "home" => "homepage_uri",
+    "code" => "source_code_uri",
+    "wiki" => "wiki_uri",
+    "mail" => "mailing_list_uri",
+  }
+
   ##
   # Default configuration values for .hoerc. Plugins should populate
   # this on load.
@@ -243,24 +254,6 @@ class Hoe
 
   attr_accessor :test_globs
 
-  ##
-  # Deprecated: Optional: The url(s) of the project. (can be array).
-  # Auto-populates to a list of urls read from the beginning of
-  # README.txt.
-  #
-
-  def url
-    warn "NOTE: Hoe#url is deprecated, use urls. It will be removed on or after 2012-06-01."
-    warn "Used from #{caller.first}"
-    @url
-  end
-
-  def url=o # :nodoc:
-    warn "NOTE: Hoe#url= is deprecated, use urls=. It will be removed on or after 2012-06-01."
-    warn "Used from #{caller.first}"
-    @url=o
-  end
-
   ##
   # Optional: The urls of the project. This can be an array or
   # (preferably) a hash. Auto-populates to the urls read from the
@@ -275,11 +268,22 @@ class Hoe
 
   attr_accessor :version
 
+  @include_dirs = []
+
+  mc = (class << self; self; end)
+
+  ##
+  # Optional: Extra directories to use (eg for test runs). See
+  # +Hoe.add_include_dirs+.
+
+  mc.send :attr_accessor, :include_dirs
+
   ##
   # Add extra dirs to both $: and RUBY_FLAGS (for test runs and rakefile deps)
 
   def self.add_include_dirs(*dirs)
     dirs = dirs.flatten
+    include_dirs.concat dirs
     $:.unshift(*dirs)
     s = File::PATH_SEPARATOR
     RUBY_FLAGS.sub!(/-I/, "-I#{dirs.join(s)}#{s}")
@@ -302,7 +306,7 @@ class Hoe
     @loaded ||= {}
     @files  ||= Gem.find_files "hoe/*.rb"
 
-    @files.reverse.each do |path|
+    @files.reverse.each do |path| # reverse so first one wins
       @found[File.basename(path, ".rb").intern] = path
     end
 
@@ -471,7 +475,7 @@ class Hoe
 
     case name
     when "hoe" then
-      dependency "rake", [">= 0.8", "< 13.0"]
+      # do nothing? these deps are already in the hoe spec in the Rakefile
     else
       version = VERSION.split(/\./).first(2).join(".")
       dependency "hoe", "~> #{version}", :development
@@ -521,14 +525,7 @@ class Hoe
       s.version              = version if version
       s.summary              = summary
       s.email                = email
-      s.homepage             = case urls
-                               when Hash then
-                                 urls["home"] || urls.values.first
-                               when Array then
-                                 urls.first
-                               else
-                                 warn "** Unknown urls format: #{urls.inspect}"
-                               end
+      s.homepage             = urls["home"] || urls.values.first
       s.description          = description
       s.files                = manifest
       s.executables          = s.files.grep(/^bin/) { |f| File.basename(f) }
@@ -536,6 +533,9 @@ class Hoe
       s.require_paths        = dirs unless dirs.empty?
       s.rdoc_options         = ["--main", readme_file]
       s.post_install_message = post_install_message
+      s.metadata             = (urls.keys & URLS_TO_META_MAP.keys).map { |name|
+        [URLS_TO_META_MAP[name], urls[name]]
+      }.to_h
 
       missing "Manifest.txt" if s.files.empty?
 
@@ -647,8 +647,8 @@ class Hoe
       self.history_file = manifest.grep(/^History\./).first
     end
 
-    self.history_file ||= Dir.glob("History.{txt,md}").first || "History.txt"
-    self.readme_file  ||= Dir.glob("README.{txt,md}").first || "README.txt"
+    self.history_file ||= Dir.glob("History.{rdoc,txt,md}").first || "History.txt"
+    self.readme_file  ||= Dir.glob("README.{rdoc,txt,md}").first || "README.txt"
 
     abort "Hoe.new {...} removed. Switch to Hoe.spec." if block_given?
   end
@@ -656,19 +656,24 @@ class Hoe
   ##
   # Intuit values from the readme and history files.
 
-  def intuit_values
-    header_re = /^((?:=+|#+) .*)$/
-    readme    = File.read_utf(readme_file).split(header_re)[1..-1] rescue ""
+  def intuit_values input
+    readme = input
+               .lines
+               .chunk { |l| l[/^(?:=+|#+)/] || "" }
+               .map(&:last)
+               .each_slice(2)
+               .map { |k, v|
+                  kp = k.join
+                  kp = kp.strip.chomp(":").split.last.downcase if k.size == 1
+                  [kp, v.join.strip]
+                }
+               .to_h
 
     unless readme.empty? then
-      sections = Hash[*readme.map { |s|
-        s =~ /^[=#]/ ? s.strip.downcase.chomp(":").split.last : s.strip
-      }]
-      desc     = sections.values_at(*description_sections).join("\n\n")
+      desc     = readme.values_at(*description_sections).join("\n\n")
       summ     = desc.split(/\.\s+/).first(summary_sentences).join(". ")
-      urls     = parse_urls(readme[1])
 
-      self.urls        ||= urls
+      self.urls        ||= parse_urls(readme.values.first)
       self.description ||= desc
       self.summary     ||= summ
     else
@@ -708,7 +713,10 @@ class Hoe
     if lines.first =~ /::/ then
       Hash[lines.map { |line| line.split(/\s*::\s*/) }]
     else
-      lines
+      warn "DEPRECATED: Please switch readme to hash format for urls."
+      warn "  Only defining 'home' url."
+      warn "  This will be removed on or after 2020-10-28."
+      { "home" => lines.first }
     end
   end
 
@@ -802,7 +810,7 @@ class Hoe
 
   def post_initialize
     activate_plugin_deps
-    intuit_values
+    intuit_values File.read_utf readme_file if readme_file
     validate_fields
     define_spec
     load_plugin_tasks
